package com.example.service.house;

import cn.hutool.core.net.URLEncodeUtil;
import com.example.common.PageVo;
import com.example.common.Rsp;
import com.example.dao.house.HouseScoreRankingDao;
import com.example.pojo.bo.ScoreRankingBo;
import com.example.pojo.dto.ScoreRankingDto;
import com.example.pojo.vo.ScoreRankingVo;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.DataFormat;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.xssf.streaming.SXSSFSheet;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Stream;


@Slf4j
@Service
@RequiredArgsConstructor
public class HouseScoreRankingService {
    private final HouseScoreRankingDao houseScoreRankingDao;

    public Rsp<PageVo<ScoreRankingVo>> scoreRanking(@Valid ScoreRankingDto dto) {
        if (dto.getMap().getStatisticalMonth() == null) {
            Integer yearMonth = houseScoreRankingDao.getLatestYearMonth();
            dto.getMap().setStatisticalMonth(yearMonth);
        }
        ScoreRankingBo firstBo = houseScoreRankingDao.firstScore(dto.getMap());
        List<ScoreRankingVo> list = new ArrayList<>();
        ScoreRankingVo vo = new ScoreRankingVo(firstBo);
        vo.setCity("全省");
        vo.setChildren(list);
        List<ScoreRankingBo> secondBos = houseScoreRankingDao.secondScore(dto.getMap());
        secondBos.forEach(t -> list.add(new ScoreRankingVo(t)));
        PageVo<ScoreRankingVo> pageVo = new PageVo<>(Collections.singletonList(vo));
        return Rsp.ok(pageVo);
    }

    public void scoreRankingExport(ScoreRankingDto.ScoreRankingMapDTO dto, HttpServletResponse response) {
        String filename = "综合评价总分排名"
                + LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyyMMddHHmmss")) + ".xlsx";
        response.setHeader("Content-Disposition", "attachment;filename=" + URLEncodeUtil.encode(filename));
        response.setContentType("application/octet-stream");
        try (SXSSFWorkbook wb = new SXSSFWorkbook()) {
            writeScoreRanking(dto, wb);
            wb.write(response.getOutputStream());
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }

    private void writeScoreRanking(ScoreRankingDto.ScoreRankingMapDTO dto, SXSSFWorkbook wb) {
        DataFormat dataFormat = wb.createDataFormat();
        // 默认样式
        CellStyle baseCellStyle = wb.createCellStyle();
        baseCellStyle.setAlignment(HorizontalAlignment.CENTER);
        baseCellStyle.setVerticalAlignment(VerticalAlignment.CENTER);
        // 整数样式
        CellStyle integerCellStyle = wb.createCellStyle();
        integerCellStyle.setDataFormat(dataFormat.getFormat("#,##0"));
        integerCellStyle.setAlignment(HorizontalAlignment.RIGHT);
        // 数字样式
        CellStyle numberCellStyle = wb.createCellStyle();
        numberCellStyle.setDataFormat(dataFormat.getFormat("#,##0.00"));
        numberCellStyle.setAlignment(HorizontalAlignment.RIGHT);
        // 百分比样式
        CellStyle percentCellStyle = wb.createCellStyle();
        percentCellStyle.setDataFormat(dataFormat.getFormat("0.00%"));
        percentCellStyle.setAlignment(HorizontalAlignment.RIGHT);
        SXSSFSheet sheet = wb.createSheet();
        AtomicInteger rowIndex = new AtomicInteger(0);
        // 表头
        Row headerRow = sheet.createRow(rowIndex.getAndIncrement());
        List<String> headers = Stream.of("资产所属单位(一级)", "资产所属单位(二级)", "闲置率=出/(出+闲)",
                "可出租率=(出+闲)/(自+出+闲)", "出租欠费率=欠费合同/合同总数", "每平方单价=不含税总价/租期/面积",
                "大于万元零星维修次数", "高频(大于每月1次)维修建筑数", "总分", "排名").toList();
        int headerLength = headers.size();
        for (int i = 0; i < headerLength; i++) {
            Cell cell = headerRow.createCell(i);
            cell.setCellValue(headers.get(i));
            cell.setCellStyle(baseCellStyle);
            // 根据内容长度设置列宽
            int columnWidth = headers.get(i).length() * 256 * 2 + 256;
            sheet.setColumnWidth(i, columnWidth);
        }
        if (dto.getStatisticalMonth() == null) {
            Integer yearMonth = houseScoreRankingDao.getLatestYearMonth();
            dto.setStatisticalMonth(yearMonth);
        }
        ScoreRankingBo firstBo = houseScoreRankingDao.firstScore(dto);
        AtomicInteger columnIndex1 = new AtomicInteger(0);
        Row row1 = sheet.createRow(rowIndex.getAndIncrement());
        // 资产所属单位(一级)
        Cell firstUnitCell1 = row1.createCell(columnIndex1.getAndIncrement());
        firstUnitCell1.setCellValue("全省");
        // 资产所属单位(二级)
        Cell areaNameCell1 = row1.createCell(columnIndex1.getAndIncrement());
        // 闲置率
        Cell xzlCell1 = row1.createCell(columnIndex1.getAndIncrement());
        if (firstBo.getXzl() != null) {
            xzlCell1.setCellValue(firstBo.getXzl().doubleValue());
        }
        xzlCell1.setCellStyle(percentCellStyle);
        // 可出租率
        Cell kczlCell1 = row1.createCell(columnIndex1.getAndIncrement());
        if (firstBo.getKczl() != null) {
            kczlCell1.setCellValue(firstBo.getKczl().doubleValue());
        }
        kczlCell1.setCellStyle(percentCellStyle);
        // 出租欠费率
        Cell czqflCell1 = row1.createCell(columnIndex1.getAndIncrement());
        if (firstBo.getCzqfl() != null) {
            czqflCell1.setCellValue(firstBo.getCzqfl().doubleValue());
        }
        czqflCell1.setCellStyle(percentCellStyle);
        // 每平方单价
        Cell czdjCell1 = row1.createCell(columnIndex1.getAndIncrement());
        if (firstBo.getCzdj() != null) {
            czdjCell1.setCellValue(firstBo.getCzdj().doubleValue());
        }
        czdjCell1.setCellStyle(numberCellStyle);
        // 大于万元零星维修次数
        Cell wylxwxCell1 = row1.createCell(columnIndex1.getAndIncrement());
        if (firstBo.getWylxwx() != null) {
            wylxwxCell1.setCellValue(firstBo.getWylxwx().doubleValue());
        }
        wylxwxCell1.setCellStyle(integerCellStyle);
        // 高频(大于每月1次)维修建筑数
        Cell gpwxjzCell1 = row1.createCell(columnIndex1.getAndIncrement());
        if (firstBo.getGpwxjz() != null) {
            gpwxjzCell1.setCellValue(firstBo.getGpwxjz().doubleValue());
        }
        gpwxjzCell1.setCellStyle(integerCellStyle);
        // 总分
        Cell zfCell1 = row1.createCell(columnIndex1.getAndIncrement());
        if (firstBo.getZf() != null) {
            zfCell1.setCellValue(firstBo.getZf().doubleValue());
        }
        zfCell1.setCellStyle(numberCellStyle);
        // 排名
        Cell pmCell1 = row1.createCell(columnIndex1.getAndIncrement());
        if (firstBo.getPm() != null) {
            pmCell1.setCellValue(firstBo.getPm().doubleValue());
        }
        pmCell1.setCellStyle(integerCellStyle);
        List<ScoreRankingBo> secondBos = houseScoreRankingDao.secondScore(dto);
        // 数据
        for (ScoreRankingBo t : secondBos) {
            AtomicInteger columnIndex = new AtomicInteger(0);
            Row row = sheet.createRow(rowIndex.getAndIncrement());
            // 资产所属单位(一级)
            Cell firstUnitCell = row.createCell(columnIndex.getAndIncrement());
            // 资产所属单位(二级)
            Cell areaNameCell = row.createCell(columnIndex.getAndIncrement());
            if (t.getAreaName() != null) {
                areaNameCell.setCellValue(t.getAreaName());
            }
            // 闲置率
            Cell xzlCell = row.createCell(columnIndex.getAndIncrement());
            if (t.getXzl() != null) {
                xzlCell.setCellValue(t.getXzl().doubleValue());
            }
            xzlCell.setCellStyle(percentCellStyle);
            // 可出租率
            Cell kczlCell = row.createCell(columnIndex.getAndIncrement());
            if (t.getKczl() != null) {
                kczlCell.setCellValue(t.getKczl().doubleValue());
            }
            kczlCell.setCellStyle(percentCellStyle);
            // 出租欠费率
            Cell czqflCell = row.createCell(columnIndex.getAndIncrement());
            if (t.getCzqfl() != null) {
                czqflCell.setCellValue(t.getCzqfl().doubleValue());
            }
            czqflCell.setCellStyle(percentCellStyle);
            // 每平方单价
            Cell czdjCell = row.createCell(columnIndex.getAndIncrement());
            if (t.getCzdj() != null) {
                czdjCell.setCellValue(t.getCzdj().doubleValue());
            }
            czdjCell.setCellStyle(numberCellStyle);
            // 大于万元零星维修次数
            Cell wylxwxCell = row.createCell(columnIndex.getAndIncrement());
            if (t.getWylxwx() != null) {
                wylxwxCell.setCellValue(t.getWylxwx().doubleValue());
            }
            wylxwxCell.setCellStyle(integerCellStyle);
            // 高频(大于每月1次)维修建筑数
            Cell gpwxjzCell = row.createCell(columnIndex.getAndIncrement());
            if (t.getGpwxjz() != null) {
                gpwxjzCell.setCellValue(t.getGpwxjz().doubleValue());
            }
            gpwxjzCell.setCellStyle(integerCellStyle);
            // 总分
            Cell zfCell = row.createCell(columnIndex.getAndIncrement());
            if (t.getZf() != null) {
                zfCell.setCellValue(t.getZf().doubleValue());
            }
            zfCell.setCellStyle(numberCellStyle);
            // 排名
            Cell pmCell = row.createCell(columnIndex.getAndIncrement());
            if (t.getPm() != null) {
                pmCell.setCellValue(t.getPm().doubleValue());
            }
            pmCell.setCellStyle(integerCellStyle);
        }
    }
}
