package com.example.service.house;

import cn.hutool.core.net.URLEncodeUtil;
import com.example.common.PageVo;
import com.example.common.Rsp;
import com.example.dao.house.HouseRentInDao;
import com.example.pojo.bo.RentInUseStatBo;
import com.example.pojo.dto.RentInUseStatDto;
import com.example.pojo.po.house.RentInUseStatPo;
import com.example.pojo.vo.RentInUseStatVo;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.DataFormat;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.servlet.http.HttpServletResponse;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Stream;


@Slf4j
@Service
public class HouseUseService {
    private final HouseRentInDao houseRentInDao;

    public HouseUseService(HouseRentInDao houseRentInDao) {
        this.houseRentInDao = houseRentInDao;
    }

    public Rsp<PageVo<RentInUseStatVo>> rentInUseStat(RentInUseStatDto dto) {
        RentInUseStatPo pos = getRentInUseStatPos(dto);
        if (pos == null) {
            return Rsp.ok();
        }
        RentInUseStatVo vo = new RentInUseStatVo(pos, false);
        PageVo<RentInUseStatVo> pageVo = new PageVo<>(Collections.singletonList(vo));
        return Rsp.ok(pageVo);
    }

    private RentInUseStatPo getRentInUseStatPos(RentInUseStatDto dto) {
        RentInUseStatBo bo = new RentInUseStatBo();
        if (dto.getMap() != null && dto.getMap().getStatisticalMonth() != null) {
            bo.setYearMonth(dto.getMap().getStatisticalMonth());
        }
        if (dto.getMap() != null && StringUtils.hasText(dto.getMap().getCity()) && !"全省".equals(dto.getMap().getCity())) {
            bo.setAreaNo(dto.getMap().getCity());
        }
        if (dto.getMap() != null && StringUtils.hasText(dto.getMap().getCounty())) {
            bo.setCityNo(dto.getMap().getCounty());
        }
        // 查询三级单位统计
        List<RentInUseStatPo> pos = houseRentInDao.rentInUseStat(bo);
        if (CollectionUtils.isEmpty(pos)) {
            return null;
        }
        // 一级单位统计
        RentInUseStatPo firstUnitPo = new RentInUseStatPo();
        firstUnitPo.setAreaName("全省");
        firstUnitPo.setCityName("全省");
        LinkedHashMap<String, RentInUseStatPo> secondUnitPoMap = new LinkedHashMap<>();
        // 二级单位统计
        for (RentInUseStatPo houseAgeStatPo : pos) {
            secondUnitPoMap.putIfAbsent(houseAgeStatPo.getAreaNo(),
                    new RentInUseStatPo(houseAgeStatPo.getAreaNo(), houseAgeStatPo.getAreaName()));
            RentInUseStatPo secondUnitPo = secondUnitPoMap.get(houseAgeStatPo.getAreaNo());
            secondUnitPo.update(houseAgeStatPo);
        }
        for (RentInUseStatPo value : secondUnitPoMap.values()) {
            firstUnitPo.update(value);
        }
        if (firstUnitPo.getZj().equals(0)) {
            return null;
        }
        return firstUnitPo;
    }

    public void rentInUseStatExport(RentInUseStatDto.RentInUseStatMapDTO dto, HttpServletResponse response) {
        String filename = "房屋租入_租入用途管理"
                + LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyyMMddHHmmss")) + ".xlsx";
        response.setHeader("Content-Disposition", "attachment;filename=" + URLEncodeUtil.encode(filename));
        response.setContentType("application/octet-stream");
        try (SXSSFWorkbook wb = new SXSSFWorkbook()) {
            writeRentInUseStat(new RentInUseStatDto(dto), wb);
            wb.write(response.getOutputStream());
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }

    private void writeRentInUseStat(RentInUseStatDto dto, SXSSFWorkbook wb) {
        RentInUseStatPo firstPo = getRentInUseStatPos(dto);
        DataFormat dataFormat = wb.createDataFormat();
        // 整数样式
        CellStyle integerCellStyle = wb.createCellStyle();
        integerCellStyle.setDataFormat(dataFormat.getFormat("#,##0"));
        integerCellStyle.setAlignment(HorizontalAlignment.RIGHT);
        // 默认样式
        CellStyle baseCellStyle = wb.createCellStyle();
        baseCellStyle.setAlignment(HorizontalAlignment.CENTER);
        baseCellStyle.setVerticalAlignment(VerticalAlignment.CENTER);
        Sheet sheet = wb.createSheet();
        AtomicInteger rowIndex = new AtomicInteger(0);
        // 表头
        Row headerRow = sheet.createRow(rowIndex.getAndIncrement());
        List<String> headers = Stream.of("资产所属单位(一级)", "资产所属单位(二级)", "资产所属单位(三级)",
                "营业", "办公", "通信设施机房", "其他租入用途", "总计").toList();
        int headerLength = headers.size();
        for (int i = 0; i < headerLength; i++) {
            Cell cell = headerRow.createCell(i);
            cell.setCellValue(headers.get(i));
            cell.setCellStyle(baseCellStyle);
            // 根据内容长度设置列宽
            int columnWidth = headers.get(i).length() * 256 * 2 + 256 * 4;
            sheet.setColumnWidth(i, columnWidth);
        }
        if (firstPo == null) {
            return;
        }
        // 数据
        AtomicInteger columnIndex1 = new AtomicInteger(0);
        // 一级单位统计
        Row row1 = sheet.createRow(rowIndex.getAndIncrement());
        // 资产所属单位(一级)
        Cell firstUnitCell1 = row1.createCell(columnIndex1.getAndIncrement());
        if (firstPo.getAreaName() != null) {
            firstUnitCell1.setCellValue(firstPo.getAreaName());
        }
        // 二级单位
        Cell areaNameCell1 = row1.createCell(columnIndex1.getAndIncrement());
//        if (firstPo.getAreaName() != null) {
//            areaNameCell1.setCellValue(firstPo.getAreaName());
//        }
        // 三级单位
        Cell cityNameCell1 = row1.createCell(columnIndex1.getAndIncrement());
//        if (firstPo.getCityName() != null) {
//            cityNameCell1.setCellValue(firstPo.getCityName());
//        }
        // 营业
        Cell yyCell1 = row1.createCell(columnIndex1.getAndIncrement());
        if (firstPo.getYy() != null) {
            yyCell1.setCellValue(firstPo.getYy());
        }
        yyCell1.setCellStyle(integerCellStyle);
        // 办公
        Cell bgCell1 = row1.createCell(columnIndex1.getAndIncrement());
        if (firstPo.getBg() != null) {
            bgCell1.setCellValue(firstPo.getBg());
        }
        bgCell1.setCellStyle(integerCellStyle);
        // 通信机房
        Cell txCell1 = row1.createCell(columnIndex1.getAndIncrement());
        if (firstPo.getTx() != null) {
            txCell1.setCellValue(firstPo.getTx());
        }
        txCell1.setCellStyle(integerCellStyle);
        // 其他用途
        Cell qtCell1 = row1.createCell(columnIndex1.getAndIncrement());
        if (firstPo.getQt() != null) {
            qtCell1.setCellValue(firstPo.getQt());
        }
        qtCell1.setCellStyle(integerCellStyle);
        // 总计
        Cell zjCell1 = row1.createCell(columnIndex1.getAndIncrement());
        if (firstPo.getZj() != null) {
            zjCell1.setCellValue(firstPo.getZj());
        }
        zjCell1.setCellStyle(integerCellStyle);
        // 二级单位统计
        for (RentInUseStatPo secondPo : firstPo.getChildren()) {
            AtomicInteger columnIndex2 = new AtomicInteger(0);
            // 一级单位统计
            Row row2 = sheet.createRow(rowIndex.getAndIncrement());
            // 资产所属单位(一级)
            Cell firstUnitCell2 = row2.createCell(columnIndex2.getAndIncrement());
//            if (secondPo.getAreaName() != null) {
//                firstUnitCell2.setCellValue(secondPo.getAreaName());
//            }
            // 二级单位
            Cell areaNameCell2 = row2.createCell(columnIndex2.getAndIncrement());
            if (secondPo.getAreaName() != null) {
                areaNameCell2.setCellValue(secondPo.getAreaName());
            }
            // 三级单位
            Cell cityNameCell2 = row2.createCell(columnIndex2.getAndIncrement());
//            if (secondPo.getCityName() != null) {
//                cityNameCell2.setCellValue(secondPo.getCityName());
//            }
            // 营业
            Cell yyCell2 = row2.createCell(columnIndex2.getAndIncrement());
            if (secondPo.getYy() != null) {
                yyCell2.setCellValue(secondPo.getYy());
            }
            yyCell2.setCellStyle(integerCellStyle);
            // 办公
            Cell bgCell2 = row2.createCell(columnIndex2.getAndIncrement());
            if (secondPo.getBg() != null) {
                bgCell2.setCellValue(secondPo.getBg());
            }
            bgCell2.setCellStyle(integerCellStyle);
            // 通信机房
            Cell txCell2 = row2.createCell(columnIndex2.getAndIncrement());
            if (secondPo.getTx() != null) {
                txCell2.setCellValue(secondPo.getTx());
            }
            txCell2.setCellStyle(integerCellStyle);
            // 其他用途
            Cell qtCell2 = row2.createCell(columnIndex2.getAndIncrement());
            if (secondPo.getQt() != null) {
                qtCell2.setCellValue(secondPo.getQt());
            }
            qtCell2.setCellStyle(integerCellStyle);
            // 总计
            Cell zjCell2 = row2.createCell(columnIndex2.getAndIncrement());
            if (secondPo.getZj() != null) {
                zjCell2.setCellValue(secondPo.getZj());
            }
            zjCell2.setCellStyle(integerCellStyle);
            // 三级单位统计
            for (RentInUseStatPo thirdPo : secondPo.getChildren()) {
                AtomicInteger columnIndex3 = new AtomicInteger(0);
                Row row3 = sheet.createRow(rowIndex.getAndIncrement());
                // 资产所属单位(一级)
                Cell firstUnitCell3 = row3.createCell(columnIndex3.getAndIncrement());
//                if (thirdPo.getAreaName() != null) {
//                    firstUnitCell3.setCellValue(thirdPo.getAreaName());
//                }
                // 二级单位
                Cell areaNameCell3 = row3.createCell(columnIndex3.getAndIncrement());
//                if (thirdPo.getAreaName() != null) {
//                    areaNameCell3.setCellValue(thirdPo.getAreaName());
//                }
                // 三级单位
                Cell cityNameCell3 = row3.createCell(columnIndex3.getAndIncrement());
                if (thirdPo.getCityName() != null) {
                    cityNameCell3.setCellValue(thirdPo.getCityName());
                }
                // 营业
                Cell yyCell3 = row3.createCell(columnIndex3.getAndIncrement());
                if (thirdPo.getYy() != null) {
                    yyCell3.setCellValue(thirdPo.getYy());
                }
                yyCell3.setCellStyle(integerCellStyle);
                // 办公
                Cell bgCell3 = row3.createCell(columnIndex3.getAndIncrement());
                if (thirdPo.getBg() != null) {
                    bgCell3.setCellValue(thirdPo.getBg());
                }
                bgCell3.setCellStyle(integerCellStyle);
                // 通信机房
                Cell txCell3 = row3.createCell(columnIndex3.getAndIncrement());
                if (thirdPo.getTx() != null) {
                    txCell3.setCellValue(thirdPo.getTx());
                }
                txCell3.setCellStyle(integerCellStyle);
                // 其他用途
                Cell qtCell3 = row3.createCell(columnIndex3.getAndIncrement());
                if (thirdPo.getQt() != null) {
                    qtCell3.setCellValue(thirdPo.getQt());
                }
                qtCell3.setCellStyle(integerCellStyle);
                // 总计
                Cell zjCell3 = row3.createCell(columnIndex3.getAndIncrement());
                if (thirdPo.getZj() != null) {
                    zjCell3.setCellValue(thirdPo.getZj());
                }
                zjCell3.setCellStyle(integerCellStyle);
            }
        }
    }
}
